# Generated by Django 5.0.4 on 2024-05-01 11:43

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbportfolio", "0059_fees_unique_fees"),
    ]

    operations = [
        migrations.AlterField(
            model_name="portfolioportfoliothroughmodel",
            name="type",
            field=models.CharField(
                choices=[("PRIMARY", "Primary"), ("MODEL", "Model"), ("BENCHMARK", "Benchmark"), ("INDEX", "Index")],
                default="PRIMARY",
                verbose_name="Type",
            ),
        ),
    ]
