# Generated by Django 5.0.14 on 2025-07-30 14:25

from django.db import migrations, models
from decimal import Decimal

class Migration(migrations.Migration):

    dependencies = [
        ('wbportfolio', '0083_order_alter_trade_options_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='orderproposal',
            name='min_order_value',
            field=models.IntegerField(default=0, help_text='Minimum Order Value in the Portfolio currency', verbose_name='Minimum Order Value'),
        ),
        migrations.AddField(
            model_name='order',
            name='desired_target_weight',
            field=models.DecimalField(decimal_places=8, default=Decimal('0'),
                                      help_text='Desired Target Weight (for compliance and audit)', max_digits=9,
                                      verbose_name='Desired Target Weight'),
        ),
    ]
