# Generated by Django 4.2.11 on 2024-04-05 12:06

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wbfdm", "0017_instrument_instrument_security_index"),
        ("wbportfolio", "0057_alter_portfolio_preferred_instrument_classifications_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="PMSInstrument",
            fields=[],
            options={
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("wbfdm.instrument",),
        ),
    ]
